unit VaxPipeClientThread;

interface

uses
  System.Classes, Windows, System.SysUtils,
  System.Threading, VaxPipeClient;


type TVaxPipeClientThread = class(TThread)

  private
  m_sPipeNameZERO : String;
  m_sPipeNameONE : String;

  private
  m_hQuitWait : THandle;
  m_bQuitThread : Boolean;

  public
  constructor Create(sPipeNameZERO: String; sPipeNameONE: String);
  destructor Destroy(); override;

  private
  procedure OnPipeData(nMediaPos: Integer; aDataReadPipe: PByte; nSizeReadPipe: Integer);

  public
  procedure OnVaxPipeClientRecvPCM(nMediaPos: Integer; aDataPCM: PByte; nSizePCM: Integer); virtual;

  public
  procedure Execute; override;

end;

implementation

  //////////////////////////////////////////////////////////////////////////////
  //////////////////////////////////////////////////////////////////////////////

  constructor TVaxPipeClientThread.Create(sPipeNameZERO: String; sPipeNameONE: String);
  begin

    m_bQuitThread := false;

    m_sPipeNameZERO := sPipeNameZERO;
    m_sPipeNameONE := sPipeNameONE;

    m_hQuitWait := CreateEvent(NIL, true, false, NIL);

    inherited Create();

  end;

  destructor TVaxPipeClientThread.Destroy();
  var
    hQuitWait : THandle;
  begin

    if(m_hQuitWait = 0) Then
    begin
      Exit();
    end;

    hQuitWait := m_hQuitWait;
    m_hQuitWait := 0;

    m_bQuitThread := true;

    SetEvent(hQuitWait);
    Inherited WaitFor();

    CloseHandle(hQuitWait);
    inherited;

  end;

  //////////////////////////////////////////////////////////////////////////////
  //////////////////////////////////////////////////////////////////////////////

  procedure TVaxPipeClientThread.Execute;
  var
    aDataReadPipe : PByte;
    nSizeReadPipe : Integer;

    aWaitEvent : Array[0..3] of THandle;
    aPipeClient : Array[0..2] of TVaxPipeClient;

    nElapseMilliSec : Integer;
    nWaitCount : Integer;
    nWaitIndex : LongWord;
  begin

    ////////////////////////////////////////////////////////////////////////////

    aPipeClient[0] := TVaxPipeClient.Create();

    if aPipeClient[0].CreatePipe(m_sPipeNameZERO) = false then
      Exit();


    aPipeClient[1] := TVaxPipeClient.Create();

    if aPipeClient[1].CreatePipe(m_sPipeNameONE) = false then
      Exit();

    ////////////////////////////////////////////////////////////////////////////

    if(aPipeClient[0].BeginReadPipe(aWaitEvent[0]) = false) Then
    begin
      aPipeClient[0].ClosePipe();
      Exit();
    end;

    if(aPipeClient[1].BeginReadPipe(aWaitEvent[1]) = false) Then
    begin
      aPipeClient[1].ClosePipe();
      Exit();
    end;

    ////////////////////////////////////////////////////////////////////////////

    aWaitEvent[2] := m_hQuitWait;
    nWaitCount := 3;

    ////////////////////////////////////////////////////////////////////////////

    While true do
    begin

    ////////////////////////////////////////////////////////////////////////////

      nWaitIndex := MsgWaitForMultipleObjects(nWaitCount, aWaitEvent, False, INFINITE, 0);

      if((nWaitIndex = WAIT_FAILED) OR (m_bQuitThread = true)) Then
        break;

    ////////////////////////////////////////////////////////////////////////////

      if ((WAIT_OBJECT_0 + nWaitIndex) = 2) Then
      begin
        break;
      end;


      if ((WAIT_OBJECT_0 + nWaitIndex) = 0) Then
      begin

        if(aPipeClient[0].EndReadPipe(aDataReadPipe, nSizeReadPipe) = false) Then
          Break;

        OnPipeData(0, aDataReadPipe, nSizeReadPipe);

        if(aPipeClient[0].BeginReadPipe(aWaitEvent[0]) = false) Then
          Break;

        Continue;

      end;

      if ((WAIT_OBJECT_0 + nWaitIndex) = 1) Then
      begin

        if(aPipeClient[1].EndReadPipe(aDataReadPipe, nSizeReadPipe) = false) Then
          Break;

        OnPipeData(1, aDataReadPipe, nSizeReadPipe);

        if(aPipeClient[1].BeginReadPipe(aWaitEvent[1]) = false) Then
          Break;

        Continue;

      end;


    ////////////////////////////////////////////////////////////////////////////

    end;

    aPipeClient[0].ClosePipe();
    aPipeClient[1].ClosePipe();

    aPipeClient[0].Free();
    aPipeClient[1].Free();

  end;

  //////////////////////////////////////////////////////////////////////////////
  //////////////////////////////////////////////////////////////////////////////

  procedure TVaxPipeClientThread.OnPipeData(nMediaPos: Integer; aDataReadPipe: PByte; nSizeReadPipe: Integer);
  begin
    OnVaxPipeClientRecvPCM(nMediaPos, aDataReadPipe, nSizeReadPipe);
  end;

  procedure TVaxPipeClientThread.OnVaxPipeClientRecvPCM(nMediaPos: Integer; aDataPCM: PByte; nSizePCM: Integer);
  begin

  end;

  //////////////////////////////////////////////////////////////////////////////
  //////////////////////////////////////////////////////////////////////////////

end.
