﻿Imports System
Imports System.Collections.Generic
Imports System.Collections
Imports System.Linq
Imports System.Text

Namespace ServerREC

    '////////////////////////////////////////////////////////////////////////////////////////////////////////
    '////////////////////////////////////////////////////////////////////////////////////////////////////////

    Class cVaxUser : Inherits cVaxHashTextData

        Private Shared m_objUserList As cVaxUserList = Nothing
        Private Shared m_nRegisterUserCount As Integer = 0

        Private m_bRegistered As Boolean = False

        '////////////////////////////////////////////////////////////////////////////////////////////////////////
        '////////////////////////////////////////////////////////////////////////////////////////////////////////

        Public Sub New(ByVal sUserName As String)

            If cVaxUser.m_objUserList Is Nothing Then
                cVaxUser.m_objUserList = New cVaxUserList()
            End If

        End Sub

        '////////////////////////////////////////////////////////////////////////////////////////////////////////
        '////////////////////////////////////////////////////////////////////////////////////////////////////////

        Public Shared Sub ResetAll()
            m_nRegisterUserCount = 0
            cVaxUser.m_objUserList = New cVaxUserList()
        End Sub

        '////////////////////////////////////////////////////////////////////////////////////////////////////////
        '////////////////////////////////////////////////////////////////////////////////////////////////////////

        Private Shared Function GetUser(ByVal sUserName As String) As cVaxUser
            Return m_objUserList.GetUser(sUserName)
        End Function

        '////////////////////////////////////////////////////////////////////////////////////////////////////////
        '////////////////////////////////////////////////////////////////////////////////////////////////////////

        Public Shared Sub OnRegisterUser(ByVal sUserLogin As String, ByVal sDomain As String, ByVal sUserAgentName As String, ByVal sFromIP As String, ByVal nFromPort As Integer, ByVal nRegId As Integer)

            Dim objUser As cVaxUser = cVaxUser.GetUser(sUserLogin)

            If objUser Is Nothing Then

                cVaxServerREC.mVaxREC.RejectRegister(nRegId, 404, "Not Found")

                Dim sStatus As String = sUserLogin & " " & cVaxDefine.USER_NOT_FOUND
                cVaxServerREC.mVaxREC.GenStatusLog(sStatus)

                Return

            End If

            objUser.OnRegister(sUserLogin, nRegId)

        End Sub

        Private Sub OnRegister(ByVal sUserLogin As String, ByVal nRegId As Integer)

            cVaxServerREC.mVaxREC.AuthRegister(nRegId)

            Dim sStatus As String = sUserLogin & " " & cVaxDefine.USER_REG_TRYING
            cVaxServerREC.mVaxREC.GenStatusLog(sStatus)

        End Sub

        '////////////////////////////////////////////////////////////////////////////////////////////////////////
        '////////////////////////////////////////////////////////////////////////////////////////////////////////

        Public Shared Sub OnRegisterUserSuccess(ByVal sUserLogin As String, ByVal sFromIP As String, ByVal nFromPort As Integer, ByVal nRegId As Integer)

            Dim objUser As cVaxUser = cVaxUser.GetUser(sUserLogin)
            If objUser Is Nothing Then Return

            objUser.OnRegisterSuccess(sUserLogin)

        End Sub

        Private Sub OnRegisterSuccess(ByVal sUserLogin As String)

            If m_bRegistered = False Then
                m_nRegisterUserCount += 1
            End If

            m_bRegistered = True

            Dim sStatus As String = sUserLogin & " " & cVaxDefine.USER_REG_SUCCESS
            cVaxServerREC.mVaxREC.GenStatusLog(sStatus)

        End Sub

        '////////////////////////////////////////////////////////////////////////////////////////////////////////
        '////////////////////////////////////////////////////////////////////////////////////////////////////////

        Public Shared Sub OnRegisterUserFailed(ByVal sUserLogin As String, ByVal sFromIP As String, ByVal nFromPort As Integer, ByVal nRegId As Integer)

            Dim objUser As cVaxUser = cVaxUser.GetUser(sUserLogin)
            If objUser Is Nothing Then Return

            objUser.OnRegisterFailed(sUserLogin)

        End Sub

        Private Sub OnRegisterFailed(ByVal sUserLogin As String)

            Dim sStatus As String = sUserLogin & " " & cVaxDefine.USER_REG_FAILED
            cVaxServerREC.mVaxREC.GenStatusLog(sStatus)

        End Sub

        '////////////////////////////////////////////////////////////////////////////////////////////////////////
        '////////////////////////////////////////////////////////////////////////////////////////////////////////

        Public Shared Sub OnUnRegisterUser(ByVal sUserLogin As String)

            Dim objUser As cVaxUser = cVaxUser.GetUser(sUserLogin)
            If objUser Is Nothing Then Return

            objUser.OnUnRegister(sUserLogin)

        End Sub

        Private Sub OnUnRegister(ByVal sUserLogin As String)

            If m_bRegistered = True Then
                m_nRegisterUserCount -= 1
            End If

            m_bRegistered = False

            Dim sStatus As String = sUserLogin & " " & cVaxDefine.USER_UNREGISTER
            cVaxServerREC.mVaxREC.GenStatusLog(sStatus)

        End Sub

        '////////////////////////////////////////////////////////////////////////////////////////////////////////
        '////////////////////////////////////////////////////////////////////////////////////////////////////////

        Public Shared Function AddUser(ByVal sUserName As String, ByVal sPassword As String, ByVal sAudioCodecList As String) As Boolean

            If cVaxServerREC.mVaxREC.IsInitialized() = False OrElse (sUserName.Length = 0) Then
                Return False
            End If

            RemoveUser(sUserName)

            If Not cVaxServerREC.mVaxREC.AddUser(sUserName, sPassword, sAudioCodecList) Then
                Return False
            End If

            m_objUserList.AddUser(sUserName)
            cStoreUser.AddUser(sUserName, sPassword, sAudioCodecList)

            Return True

        End Function

        '////////////////////////////////////////////////////////////////////////////////////////////////////////
        '////////////////////////////////////////////////////////////////////////////////////////////////////////

        Public Shared Sub RemoveUser(ByVal sUserName As String)

            If cVaxServerREC.mVaxREC.IsInitialized() = False OrElse (sUserName.Length = 0) Then
                Return
            End If

            cVaxServerREC.mVaxREC.RemoveUser(sUserName)
            m_objUserList.RemoveUser(sUserName)

            cStoreUser.RemoveUser(sUserName)

        End Sub

        '////////////////////////////////////////////////////////////////////////////////////////////////////////
        '////////////////////////////////////////////////////////////////////////////////////////////////////////

        Public Shared Sub LoadUser()

            If Not cVaxServerREC.mVaxREC.IsInitialized() Then
                Return
            End If

            Dim objListUsers As ArrayList = New ArrayList()
            cStoreUser.GetUserAll(objListUsers)

            If objListUsers.Count = 0 Then Return

            Dim sPassword As String = "", sAudioCodecList As String = ""

            For Each sUserName As String In objListUsers
                cStoreUser.GetUser(sUserName, sPassword, sAudioCodecList)
                AddUser(sUserName, sPassword, sAudioCodecList)
            Next

        End Sub

        '////////////////////////////////////////////////////////////////////////////////////////////////////////
        '////////////////////////////////////////////////////////////////////////////////////////////////////////

        Public Shared Function GetRegisteredUserCount() As Integer
            Return m_nRegisterUserCount
        End Function

        '////////////////////////////////////////////////////////////////////////////////////////////////////////
        '////////////////////////////////////////////////////////////////////////////////////////////////////////

    End Class

End Namespace
