#include "StdAfx.h"
#include "Define.h"

#include "StatusLogDlg.h"
#include "VaxServerREC.h"
#include "VaxCallSession.h"
#include "VaxCallSessionList.h"

#include "VaxUser.h"
#include "VaxLine.h"

#include "StoreCOMM.h"
#include "VaxSettingDefault.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

CVaxServerREC* CVaxServerREC::m_pVaxREC = NULL;

CVaxServerREC::CVaxServerREC()
{
	CVaxServerREC::m_pVaxREC = this;
		
	m_bInitialized = FALSE;
	ResetAll(TRUE, FALSE);
}

CVaxServerREC::~CVaxServerREC()
{
	UnInitialize();
	
	CVaxServerCOM::UnLoadCOM();
	ResetAll(FALSE, TRUE);
}

////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

void CVaxServerREC::ResetAll(BOOL bAlloc, BOOL bDeAlloc)
{
	if(bDeAlloc)
	{
		delete m_pSessionList;
		m_pSessionList = NULL;
	}
		
	if(bAlloc)
	{
		m_pSessionList = new CVaxCallSessionList();
	}
	
	CVaxUser::ResetAll(bAlloc, bDeAlloc);
}

//////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////

BOOL CVaxServerREC::StartServer(LONG nListenPort)
{
	StopServer();

	CVaxServerCOM::SetLicenseKey("TRIAL-LICENSE-KEY");
		
	if(!CVaxServerCOM::Initialize(""))
		return FALSE;

	if(!CVaxServerCOM::OpenNetworkUDP("", nListenPort))
	{
		UnInitialize();
		return FALSE;
	}

	if(!CVaxServerCOM::OpenNetworkTCP("", nListenPort))
    {
		UnInitialize();
		return FALSE;
	}

	if(!CVaxServerCOM::OpenNetworkTLS("", nListenPort + 1, ""))
	{
		UnInitialize();
		return FALSE;
	}

	// CVaxServerCOM::AttackDetectScanSIP("demo.vaxvoip.com");
	// CVaxServerCOM::AttackDetectFloodSIP(1000);
	// CVaxServerCOM::AttackDetectBruteForceSIP(10, 2);

	CVaxServerCOM::SetListenPortRangeRTP(20000, 30000);
    CVaxServerCOM::AudioSessionLost(TRUE, 30); // Enable voice session lost for 30 seconds.
    
	CVaxServerCOM::AddLine("AnonymousUDP", VAX_LINE_TYPE_UDP, "", "", "", "", "", "255.255.255.255", -1, "");
	CVaxServerCOM::AddLine("AnonymousTCP", VAX_LINE_TYPE_TCP, "", "", "", "", "", "255.255.255.255", -1, "");
	CVaxServerCOM::AddLine("AnonymousTLS", VAX_LINE_TYPE_TLS, "", "", "", "", "", "255.255.255.255", -1, "");
		
	m_bInitialized = true;
	OnInitialize();
   
	return TRUE;
}

void CVaxServerREC::StopServer()
{
	m_bInitialized = FALSE;
				
	CVaxServerCOM::UnInitialize();
	ResetAll(TRUE, TRUE);
}

BOOL CVaxServerREC::IsInitialized()
{
	return m_bInitialized;
}

//////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////

BOOL CVaxServerREC::AcceptCallSession(LONG nSessionId)
{
	return CVaxServerCOM::AcceptCallSession(nSessionId, 30);
}

void CVaxServerREC::SetRecordingFolder(LPCTSTR sPath)
{
	m_sRecordingPath = sPath;
}

CString CVaxServerREC::GetRecordingFolder()
{
	return m_sRecordingPath;
}

//////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////

BOOL CVaxServerREC::RecordWaveStartToCallSession(LONG nSessionId, LPCTSTR sFileName)
{
	return CVaxServerCOM::RecordWaveStartToCallSession(nSessionId, sFileName, VAX_WAVE_REC_TYPE_STEREO_PCM);
}

//////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////

void CVaxServerREC::OnCallSessionCreated(LONG nSessionId, LONG nReasonCode)
{
	CVaxCallSession* pCallSession = new CVaxCallSession(); 
	m_pSessionList->AddSession(nSessionId, pCallSession);

	pCallSession->OnCallSessionCreated(nSessionId, nReasonCode);
}

void CVaxServerREC::OnCallSessionClosed(LONG nSessionId, LONG nReasonCode)
{
	CVaxCallSession* pCallSession = m_pSessionList->GetSession(nSessionId);
	if (pCallSession == NULL) return;

	pCallSession->OnCallSessionClosed(nSessionId, nReasonCode);

	m_pSessionList->RemoveSession(nSessionId);
}

void CVaxServerREC::OnCallSessionAccessAudioPCM(LONG nSessionId, LONG nAccessType, char* pDataPCM, LONG nSizePCM, LONG nTypePCM)
{
	CVaxCallSession* pCallSession = m_pSessionList->GetSession(nSessionId);
	if (pCallSession == NULL) return;

	pCallSession->OnCallSessionAccessAudioPCM(nSessionId, nAccessType, pDataPCM, nSizePCM, nTypePCM);
}

//////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////

void CVaxServerREC::OnIncomingCall(LONG nSessionId, LPCTSTR sCallerName, LPCTSTR sCallerId, LPCTSTR sCalleeName, LPCTSTR sCalleeId, LPCTSTR sDialNo, LONG nFromPeerType, LPCTSTR sFromPeerName, LPCTSTR sUserAgent, LPCTSTR sRecXML, LPCTSTR sFromIP, UINT nFromPort)
{
	CVaxCallSession* pCallSession = m_pSessionList->GetSession(nSessionId);
	if(pCallSession == NULL) return;
	
	pCallSession->OnIncomingCall(nSessionId, sCallerName, sCallerId, sCalleeName, sCalleeId, sDialNo, nFromPeerType, sFromPeerName, sUserAgent, sRecXML, sFromIP, nFromPort);
}

void CVaxServerREC::OnCallSessionConnected(LONG nSessionId)
{
	CVaxCallSession* pCallSession = m_pSessionList->GetSession(nSessionId);
	if(pCallSession == NULL) return;

	pCallSession->OnCallSessionConnected(nSessionId);
}

//////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////

void CVaxServerREC::OnRegisterUser(LPCTSTR sUserName, LPCTSTR sDomain, LPCTSTR sUserAgent, LPCTSTR sFromIP, LONG nFromPort, LONG nRegId)
{
	CVaxUser::OnRegisterUser(sUserName, sDomain, sUserAgent, sFromIP, nFromPort, nRegId);
}

void CVaxServerREC::OnRegisterUserSuccess(LPCTSTR sUserLogin, LPCTSTR sFromIP, LONG nFromPort, LONG nRegId)
{
	CVaxUser::OnRegisterUserSuccess(sUserLogin, sFromIP, nFromPort, nRegId);
}

void CVaxServerREC::OnRegisterUserFailed(LPCTSTR sUserLogin, LPCTSTR sFromIP, LONG nFromPort, LONG nRegId)
{
	CVaxUser::OnRegisterUserFailed(sUserLogin, sFromIP, nFromPort, nRegId);
}

void CVaxServerREC::OnUnRegisterUser(LPCTSTR sUserLogin)
{
	CVaxUser::OnUnRegisterUser(sUserLogin);
}

///////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////

void CVaxServerREC::OnLineRegisterTrying(LPCTSTR sLineName)
{
	CVaxLine::OnLineRegisterTrying(sLineName);
}

void CVaxServerREC::OnLineRegisterFailed(LPCTSTR sLineName, LONG nStatusCode, LPCTSTR sReasonPhrase)
{
	CVaxLine::OnLineRegisterFailed(sLineName, nStatusCode, sReasonPhrase);
}

void CVaxServerREC::OnLineRegisterSuccess(LPCTSTR sLineName)
{
	CVaxLine::OnLineRegisterSuccess(sLineName);
}

//////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////

void CVaxServerREC::GenStatusLog(LPCTSTR sStatus)
{
	OnStatusLog(sStatus);
}

void CVaxServerREC::OnVaxErrorLog(LPCTSTR sFuncName, LONG nErrorCode, LPCTSTR sErrorMsg)
{
	CString sMsg;
	sMsg.Format("[Method: %s] [Error Msg: %s] [Code: %d]", sFuncName, sErrorMsg, nErrorCode);

	OnStatusLog(sMsg);
}

void CVaxServerREC::OnCallSessionErrorLog(LONG nSessionId, LONG nErrorCode, LPCTSTR sErrorMsg)
{
	CString sMsg;
	sMsg.Format("[Session: %d] [Error Msg: %s] [Code: %d]", nSessionId, sErrorMsg, nErrorCode);

	OnStatusLog(sMsg);
}

/////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////

void CVaxServerREC::OnInitialize()
{
	if(!CVaxSettingDefault::LoadUserDefault())
	{
		CVaxUser::LoadUser();
	}
	
	if(!CVaxSettingDefault::LoadLineDefault())
	{
		CVaxLine::LoadLine();
	}
}

void CVaxServerREC::OnStatusLog(LPCTSTR sStatus)
{
	m_pStatusLogDlg->OnStatusLog(sStatus);
}