#include "stdafx.h"
#include "VaxServerREC.h"

#include "VaxUserList.h"
#include "VaxUser.h"

#include "StoreUser.h"
#include "VaxCallSession.h"

//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////

CVaxUserList* CVaxUser::m_pUserList = NULL;
LONG CVaxUser::m_nRegisterUserCount = 0;

//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////

CVaxUser::CVaxUser(LPCTSTR sUserName)
{
	if(CVaxUser::m_pUserList == NULL)
		CVaxUser::m_pUserList = new CVaxUserList();
	
	m_bRegistered = FALSE;
}

CVaxUser::~CVaxUser(void)
{
}

//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////

void CVaxUser::ResetAll(BOOL bAlloc, BOOL bDeAlloc)
{
	m_nRegisterUserCount = 0;
	
	if(bDeAlloc)
	{
		if(CVaxUser::m_pUserList != NULL)
		{
			delete CVaxUser::m_pUserList;
			CVaxUser::m_pUserList = NULL;
		}
	}

	if(bAlloc)
		CVaxUser::m_pUserList = new CVaxUserList();
}

//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////

CVaxUser* CVaxUser::GetUser(LPCTSTR sUserName)
{
	return m_pUserList->GetUser(sUserName);
}

//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////

void CVaxUser::OnRegisterUser(LPCTSTR sUserName, LPCTSTR sDomain, LPCTSTR sUserAgent, LPCTSTR sFromIP, LONG nFromPort, LONG nRegId)
{
	CVaxUser* pUser = CVaxUser::GetUser(sUserName);
	
	if(pUser == NULL)
	{
		CVaxServerREC::m_pVaxREC->RejectRegister(nRegId, 404, "Not Found");

		CString sStatus = sUserName + CString(" ") + USER_NOT_FOUND;
		CVaxServerREC::m_pVaxREC->GenStatusLog(sStatus);

		return;
	}
				
	pUser->OnRegister(sUserName, nRegId);
}

void CVaxUser::OnRegister(LPCTSTR sUserName, LONG nRegId)
{
	CVaxServerREC::m_pVaxREC->AuthRegister(nRegId);

	CString sStatus = sUserName + CString(" ") + USER_REG_TRYING;
    CVaxServerREC::m_pVaxREC->GenStatusLog(sStatus);
}

//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////

void CVaxUser::OnRegisterUserSuccess(LPCTSTR sUserName, LPCTSTR sFromIP, LONG nFromPort, LONG nRegId)
{
	CVaxUser* pUser = CVaxUser::GetUser(sUserName);
	if(pUser == NULL) return;
			
	pUser->OnRegisterSuccess(sUserName);
}

void CVaxUser::OnRegisterSuccess(LPCTSTR sUserName)
{	
	 if (m_bRegistered == FALSE)
		m_nRegisterUserCount++;

	m_bRegistered = TRUE;

	CString sStatus = sUserName + CString(" ") + USER_REG_SUCCESS;
    CVaxServerREC::m_pVaxREC->GenStatusLog(sStatus);
}

//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////

void CVaxUser::OnRegisterUserFailed(LPCTSTR sUserName, LPCTSTR sFromIP, LONG nFromPort, LONG nRegId)
{
	CVaxUser* pUser = CVaxUser::GetUser(sUserName);
	if(pUser == NULL) return;

	pUser->OnRegisterFailed(sUserName);
}

void CVaxUser::OnRegisterFailed(LPCTSTR sUserName)
{
	CString sStatus = sUserName + CString(" ") + USER_REG_FAILED;
    CVaxServerREC::m_pVaxREC->GenStatusLog(sStatus);
}

//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////

void CVaxUser::OnUnRegisterUser(LPCTSTR sUserName)
{
	CVaxUser* pUser = CVaxUser::GetUser(sUserName);
	if(pUser == NULL) return;
	
	pUser->OnUnRegister(sUserName);
}

void CVaxUser::OnUnRegister(LPCTSTR sUserName)
{	
	if (m_bRegistered == TRUE)
		m_nRegisterUserCount--;

	m_bRegistered = FALSE;

	CString sStatus = sUserName + CString(" ") + USER_UNREGISTER;
    CVaxServerREC::m_pVaxREC->GenStatusLog(sStatus);
}

//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////

BOOL CVaxUser::AddUser(LPCTSTR sUserName, LPCTSTR sPassword, LPCTSTR sAudioCodecList)
{
	if(!CVaxServerREC::m_pVaxREC->IsInitialized() || CString(sUserName).IsEmpty())
		return FALSE;
	
	RemoveUser(sUserName);
	
	if (!CVaxServerREC::m_pVaxREC->AddUser(sUserName, sPassword, sAudioCodecList))
    {
		return FALSE;
	}

	
    CVaxUser::m_pUserList->AddUser(sUserName);

    CStoreUser::AddUser(sUserName, sPassword, sAudioCodecList);
	return TRUE;
}

//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////

void CVaxUser::RemoveUser(LPCTSTR sUserName)
{
	if(!CVaxServerREC::m_pVaxREC->IsInitialized() || CString(sUserName).IsEmpty())
		return;
		
	CVaxServerREC::m_pVaxREC->RemoveUser(sUserName);
    CVaxUser::m_pUserList->RemoveUser(sUserName);

    CStoreUser::RemoveUser(sUserName);
}

//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////

void CVaxUser::LoadUser()
{
	if(!CVaxServerREC::m_pVaxREC->IsInitialized())
		return;

	CStringList objListUsers;
	CStoreUser::GetUserAll(objListUsers);

	if(objListUsers.IsEmpty())
		return;

	CString sUserName, sPassword, sAudioCodecList;
	    	   		
	while(!objListUsers.IsEmpty())
	{
		sUserName = objListUsers.RemoveHead();
		
		CStoreUser::GetUser(sUserName, sPassword, sAudioCodecList);
		AddUser(sUserName, sPassword, sAudioCodecList);
	}
}

//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////

LONG CVaxUser::GetRegisteredUserCount()
{
	return CVaxUser::m_nRegisterUserCount;
}

