using System;
using System.Collections;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Diagnostics;

using ServerREC;

namespace ServerREC
{
    public partial class GeneralForm : Form
    {
        private const String STATUS_MSG_SERVER_RUNNING = "SIP SERVER IS RUNNING.";
        private const String STATUS_MSG_SERVER_NOT_RUNNING = "SIP SERVER IS NOT RUNNING.";

        private const String START_BUTTON_TITLE = "Start";
        private const String STOP_BUTTON_TITLE = "Stop";
                
        Boolean m_bServerRunning;
    
        private PerformanceCounter m_objUsageCPU = new PerformanceCounter("Processor", "% Processor Time", "_Total");

        public GeneralForm()
        {
            InitializeComponent();
                    
            LabelServerStatus.Text = STATUS_MSG_SERVER_NOT_RUNNING;
            BtnStartServerSIP.Text = START_BUTTON_TITLE;

            m_bServerRunning = false;
        }
                
        private void GeneralForm_Load(object sender, EventArgs e)
        {
            int nListenPortSIP = cStoreCOMM.GetListenPortSIP();
            EditListenPort.Text = nListenPortSIP.ToString();
            
            UpdateStats();
            TimerGeneral.Enabled = true;
        }

        private void BtnStartServerSIP_Click(object sender, EventArgs e)
        {
            if (m_bServerRunning)
            {
                cVaxServerREC.mVaxREC.StopServer();
                TimerGeneral.Enabled = false;

                LabelServerStatus.Text = STATUS_MSG_SERVER_NOT_RUNNING;
                BtnStartServerSIP.Text = START_BUTTON_TITLE;

                m_bServerRunning = false;
                return;
            }

            Boolean bStatusRun = cVaxServerREC.mVaxREC.StartServer(Convert.ToInt32(EditListenPort.Text));

            if( bStatusRun == false )
            {
                LabelServerStatus.Text = "UNABLE TO START SIP SERVER (SEE ERROR LOG).";
                return;
            }

            cStoreCOMM.SetListenPortSIP(Convert.ToInt32(EditListenPort.Text));
            cVaxServerREC.mVaxREC.SetRecordingFolder(txtFolderPath.Text);
            
            TimerGeneral.Enabled = true;
                        
            LabelServerStatus.Text = STATUS_MSG_SERVER_RUNNING;
            BtnStartServerSIP.Text = STOP_BUTTON_TITLE;

            m_bServerRunning = true;
        }
        
                
        private void TimerGeneral_Tick(object sender, EventArgs e)
        {
            UpdateStats();
        }

        private void UpdateStats()
        {
            LabelTotalCalls.Text = cVaxServerREC.mVaxREC.GetCallCount().ToString();
            LabelTotalUsers.Text = cVaxUser.GetRegisteredUserCount().ToString();

            LabelUsageCPU.Text = GetUsageCPU().ToString();
            LabelUsageMEM.Text = GetAvailableMEM().ToString();
        }

        public void OnSetActive()
        {
          
        }

        public long GetAvailableMEM()
        {
            PerformanceCounter objUsageMEM = new PerformanceCounter("Memory", "Available KBytes");
            return (long) objUsageMEM.NextValue();

        }

        public long GetUsageCPU()
        {
            return (long) m_objUsageCPU.NextValue();
        }

    }
}