using System;
using System.Collections;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Collections.Generic;

using ServerREC;

namespace ServerREC
{
    public partial class LineForm : Form
    {
        public LineForm()
        {
            InitializeComponent();
        }
 
        private void BtnAddLine_Click(object sender, EventArgs e)
        {
            if(AddLineChecks() == false)
                return;

            if(ComboLine.FindStringExact(ComboLine.Text) != -1)
            {
                if(MessageBox.Show("Line already exists, do you want to continue?", "VaxVoIP", MessageBoxButtons.YesNo, MessageBoxIcon.Exclamation) == DialogResult.No)
                {
                    return;
                }
            }
                        
            ///////////////////////////////////////////////////////////////////////////////

            String sCodec = GetAudioCodecUI();
            int nLineType = GetLineTypeUI();

            ///////////////////////////////////////////////////////////////////////////////

            if(cVaxLine.AddLine(ComboLine.Text, nLineType, EditAuthUser.Text, EditPassword.Text, EditServerAddr.Text, Convert.ToInt32(EditServerPort.Text), sCodec, CheckBoxRegLine.Checked) == false)
            {
                MessageBox.Show("Line could not be added or Please start the server first.", "VaxVoIP", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            if(!ComboLine.Items.Contains(ComboLine.Text))
                ComboLine.Items.Add(ComboLine.Text);

            MessageBox.Show("Line added/updated successfully.", "VaxVoIP", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
        }
              
        
        private void BtnRemoveLine_Click(object sender, EventArgs e)
        {
            if(ComboLine.Text.Length == 0)
            {
                MessageBox.Show("Please enter line name.", "VaxVoIP", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                return;
            }

            int nIndex = ComboLine.FindStringExact(ComboLine.Text);
            if(nIndex == -1) return;

            cVaxLine.RemoveLine(ComboLine.Text);
            ComboLine.Items.RemoveAt(nIndex);

            ClearDlgUI();
            
            if (ComboLine.Items.Count > 0)
                ComboLine.SelectedIndex = 0;
                        
            ShowLineDetails();
            
            MessageBox.Show("Line removed successfully!", "VaxVoIP", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
        }

        //////////////////////////////////////////////////////////////////////////
        //////////////////////////////////////////////////////////////////////////

        private void ComboLine_SelectedIndexChanged(object sender, EventArgs e)
        {
            ShowLineDetails();
        }

        private void BtnClearLine_Click(object sender, EventArgs e)
        {
            ClearDlgUI();
        }
        
        //////////////////////////////////////////////////////////////////////////
        //////////////////////////////////////////////////////////////////////////

        public void OnSetActive()
        {
            AddLines();
            ShowLineDetails();
        }

        //////////////////////////////////////////////////////////////////////////
        //////////////////////////////////////////////////////////////////////////

        private void ShowLineDetails()
        {
            String sLineName = ComboLine.Text;
            
            if (sLineName.Length == 0)
                return;

            String sAuthUser = "", sAuthPwd = "", sServerAddr = "";
            String sAudioCodecList = "";
            
            int nServerPort = cVaxServerCOM.SIP_DEFAULT_PORT, nLineType = cVaxServerCOM.VAX_LINE_TYPE_UDP;
            Boolean bRegister = false;

            cStoreLine.GetLine(sLineName, ref nLineType, ref sAuthUser, ref sAuthPwd, ref sServerAddr, ref nServerPort, ref sAudioCodecList, ref bRegister);
            UpdateDlgUI(sLineName, nLineType, sAuthUser, sAuthPwd, sServerAddr, nServerPort, sAudioCodecList, bRegister);
        }

        //////////////////////////////////////////////////////////////////////////
        //////////////////////////////////////////////////////////////////////////

        private void UpdateDlgUI(String sLineName, int nLineType, String sAuthUser, String sAuthPwd, String sServerAddr, int nServerPort, String sAudioCodecList, Boolean bRegister)
        {
            if(sLineName.Length == 0)
                return;
         
            EditAuthUser.Text = sAuthUser;
            EditPassword.Text = sAuthPwd;

            EditServerAddr.Text = sServerAddr;
            EditServerPort.Text = nServerPort.ToString();

            CheckBoxRegLine.Checked = bRegister;

            UpdateLineTypeUI(nLineType);
            UpdateAudioCodecUI(sAudioCodecList);
        }
                
        //////////////////////////////////////////////////////////////////////////
        //////////////////////////////////////////////////////////////////////////

        private void UpdateAudioCodecUI(String sAudioCodecList)
        {
            ListBoxEnableCodec.Items.Clear();
            ListBoxDisableCodec.Items.Clear();

            for (int nCount = 0; nCount < cVaxServerCOM.VAX_CODEC_TOTAL_COUNT; nCount++)
                ListBoxDisableCodec.Items.Add(cCodec.GetCodecName(nCount));

            String sCodecName = "";

            for (int nCount = 0; nCount < sAudioCodecList.Length; nCount++)
            {
                sCodecName = cCodec.GetCodecName(sAudioCodecList[nCount]);

                ListBoxEnableCodec.Items.Add(sCodecName);
                ListBoxDisableCodec.Items.Remove(sCodecName);
            }
        }

        private String GetAudioCodecUI()
        {
            String sCodec = "";

            for (int nCount = 0; nCount < ListBoxEnableCodec.Items.Count; nCount++)
                sCodec += cCodec.GetCodecNo((String)ListBoxEnableCodec.Items[nCount]);

            return sCodec;
        }

        //////////////////////////////////////////////////////////////////////////
        //////////////////////////////////////////////////////////////////////////

        private void UpdateLineTypeUI(int nLineType)
        {
            if (nLineType == cVaxServerCOM.VAX_LINE_TYPE_UDP)
                RadioLineTypeUDP.Checked = true;

            else if (nLineType == cVaxServerCOM.VAX_LINE_TYPE_TCP)
                RadioLineTypeTCP.Checked = true;

            else if (nLineType == cVaxServerCOM.VAX_LINE_TYPE_TLS)
                RadioLineTypeTLS.Checked = true;

            else
                RadioLineTypeUDP.Checked = true;
        }

        private int GetLineTypeUI()
        {
            if (RadioLineTypeUDP.Checked == true)
            {
                return cVaxServerCOM.VAX_LINE_TYPE_UDP;
            }

            if (RadioLineTypeTCP.Checked == true)
            {
                return cVaxServerCOM.VAX_LINE_TYPE_TCP;
            }

            if (RadioLineTypeTLS.Checked == true)
            {
                return cVaxServerCOM.VAX_LINE_TYPE_TLS;
            }

            return cVaxServerCOM.VAX_LINE_TYPE_UDP;
        }

        //////////////////////////////////////////////////////////////////////////
        //////////////////////////////////////////////////////////////////////////

        private void ClearDlgUI()
        {
            ComboLine.Text = "";

            EditAuthUser.Text = "";
            EditPassword.Text = "";

            CheckBoxRegLine.Checked = false;

            EditServerAddr.Text = "";
            EditServerPort.Text = Convert.ToString(cVaxServerCOM.SIP_DEFAULT_PORT);
            
            RadioLineTypeUDP.Checked = true;

            ListBoxEnableCodec.Items.Clear();
            ListBoxDisableCodec.Items.Clear();

            ListBoxEnableCodec.Items.Add(cCodec.AUDIO_CODEC_NAME_G711U);
            ListBoxEnableCodec.Items.Add(cCodec.AUDIO_CODEC_NAME_G711A);
            ListBoxEnableCodec.Items.Add(cCodec.AUDIO_CODEC_NAME_ILBC);
            ListBoxEnableCodec.Items.Add(cCodec.AUDIO_CODEC_NAME_GSM610);
            ListBoxEnableCodec.Items.Add(cCodec.AUDIO_CODEC_NAME_G729);
        }

        //////////////////////////////////////////////////////////////////////////
        //////////////////////////////////////////////////////////////////////////

        private Boolean AddLineChecks()
        {
            if (ComboLine.Text.Length == 0)
            {
                MessageBox.Show("Please enter Line Name.", "VaxVoIP", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                return false;
            }

            if (ListBoxEnableCodec.Items.Count == 0)
            {
                MessageBox.Show("Please select a codec.", "VaxVoIP", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                return false;
            }

            return true;
        }

        //////////////////////////////////////////////////////////////////////////
        //////////////////////////////////////////////////////////////////////////

        private void BtnDisableCodec_Click(object sender, EventArgs e)
        {
            if (ListBoxEnableCodec.SelectedItems.Count == 0) return;

            ListBoxDisableCodec.Items.Add(ListBoxEnableCodec.SelectedItem.ToString());
            ListBoxEnableCodec.Items.Remove(ListBoxEnableCodec.SelectedItem);
        }

        private void BtnEnableCodec_Click(object sender, EventArgs e)
        {
            if (ListBoxDisableCodec.SelectedItems.Count == 0) return;

            ListBoxEnableCodec.Items.Add(ListBoxDisableCodec.SelectedItem.ToString());
            ListBoxDisableCodec.Items.Remove(ListBoxDisableCodec.SelectedItem);
        }

        private void AddLines()
        {
            ComboLine.Items.Clear();
                    
            ArrayList objListLines = new ArrayList();
            cStoreLine.GetLineNameAll(ref objListLines);

            if (objListLines.Count <= 0)
            {
                return;
            }

            for (int nCount = 0; nCount < objListLines.Count; nCount++)
                ComboLine.Items.Add(objListLines[nCount]);

            if (ComboLine.Items.Count > 0)
                ComboLine.SelectedIndex = 0;
        }
    }
}