using System;
using System.Collections;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using ServerREC;

namespace ServerREC
{
    public partial class LoginForm : Form
    {
        //////////////////////////////////////////////////////////////////////////
        //////////////////////////////////////////////////////////////////////////
                       
        public LoginForm()
        {
            InitializeComponent();
        }
               
        //////////////////////////////////////////////////////////////////////////
        //////////////////////////////////////////////////////////////////////////

        private void BtnAddLogin_Click(object sender, EventArgs e)
        {
            if(AddLoginChecks() == false)
                return;

            if(ComboLogin.FindStringExact(ComboLogin.Text) != -1)
            {
                if(MessageBox.Show("Login already exists, do you want to continue?", "VaxVoIP", MessageBoxButtons.YesNo, MessageBoxIcon.Exclamation) == DialogResult.No)
                {
                    return;
                }
            }
                        
            ////////////////////////////////////////////////////////////////////////////////////////////

            String sCodec = GetAudioCodecUI();
            
            ////////////////////////////////////////////////////////////////////////////////////////////

            if (!cVaxUser.AddUser(ComboLogin.Text, EditPassword.Text, sCodec))
            {
                MessageBox.Show("Login could not be created or Please start the server first.", "VaxVoIP", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            ////////////////////////////////////////////////////////////////////////////////////////////

            if(!ComboLogin.Items.Contains(ComboLogin.Text))
                ComboLogin.Items.Add(ComboLogin.Text);

            MessageBox.Show("Login added/updated successfully.", "VaxVoIP", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
        }

        //////////////////////////////////////////////////////////////////////////
        //////////////////////////////////////////////////////////////////////////

        private void BtnRemoveLogin_Click(object sender, EventArgs e)
        {
            if (ComboLogin.Text.Length == 0)
            {
                MessageBox.Show("Please enter a login name.", "VaxVoIP", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                return;
            }

            int nIndex = ComboLogin.FindStringExact(ComboLogin.Text);
            if (nIndex == -1) return;

            cVaxUser.RemoveUser(ComboLogin.Text);
            ComboLogin.Items.RemoveAt(nIndex);

            ClearDlgUI();

            if (ComboLogin.Items.Count > 0)
                ComboLogin.SelectedIndex = 0;

            ShowLoginDetails();

            MessageBox.Show("Login removed successfully!", "VaxVoIP", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
        }
                
        //////////////////////////////////////////////////////////////////////////
        //////////////////////////////////////////////////////////////////////////

        private void ComboLogin_SelectedIndexChanged(object sender, EventArgs e)
        {
            ShowLoginDetails();
        }

        private void BtnClearLogin_Click(object sender, EventArgs e)
        {
            ClearDlgUI();
        }
                
        //////////////////////////////////////////////////////////////////////////
        //////////////////////////////////////////////////////////////////////////

        private void LoadLoginAll()
        {
            ComboLogin.Items.Clear();

            ArrayList objListUsers = new ArrayList();
            cStoreUser.GetUserAll(ref objListUsers);

            if(objListUsers.Count <= 0) return;

            for (int nIndex = 0; nIndex < objListUsers.Count; nIndex++)
                ComboLogin.Items.Add(objListUsers[nIndex]);

            if(ComboLogin.Items.Count > 0)
                ComboLogin.SelectedIndex = 0;
        }
        
        //////////////////////////////////////////////////////////////////////////
        //////////////////////////////////////////////////////////////////////////
   
        public void OnSetActive()
        {
            LoadLoginAll();
            ShowLoginDetails();
        }

        //////////////////////////////////////////////////////////////////////////
        //////////////////////////////////////////////////////////////////////////

        private void ShowLoginDetails()
        {
            String sLogin = ComboLogin.Text;
            
            if(sLogin.Length == 0)
                return;

            String sPassword = "", sAudioCodecList = "";
                                    
            cStoreUser.GetUser(sLogin, ref sPassword, ref sAudioCodecList);
            UpdateDlgUI(sLogin, sPassword, sAudioCodecList);
        }

        //////////////////////////////////////////////////////////////////////////
        //////////////////////////////////////////////////////////////////////////

        private void UpdateDlgUI(String sLogin, String sPassword, String sAudioCodecList)
        {
            if(sLogin.Length == 0)
                return;

            EditPassword.Text = sPassword;
            UpdateAudioCodecUI(sAudioCodecList);
        }

        //////////////////////////////////////////////////////////////////////////
        //////////////////////////////////////////////////////////////////////////

        private void UpdateAudioCodecUI(String sAudioCodecList)
        {
            ListBoxEnableCodec.Items.Clear();
            ListBoxDisableCodec.Items.Clear();

            for (int nCount = 0; nCount < cVaxServerCOM.VAX_CODEC_TOTAL_COUNT; nCount++)
                ListBoxDisableCodec.Items.Add(cCodec.GetCodecName(nCount));

            String sCodecName = "";

            for (int nCount = 0; nCount < sAudioCodecList.Length; nCount++)
            {
                sCodecName = cCodec.GetCodecName(sAudioCodecList[nCount]);

                ListBoxEnableCodec.Items.Add(sCodecName);
                ListBoxDisableCodec.Items.Remove(sCodecName);
            }
        }
                
        private String GetAudioCodecUI()
        {
            String sCodec = "";

            for (int nCount = 0; nCount < ListBoxEnableCodec.Items.Count; nCount++)
                sCodec += cCodec.GetCodecNo((String)ListBoxEnableCodec.Items[nCount]);

            return sCodec;
        }

        //////////////////////////////////////////////////////////////////////////
        //////////////////////////////////////////////////////////////////////////

        private void ClearDlgUI()
        {
            ComboLogin.Text = "";
            EditPassword.Text = "";
                        
            ListBoxEnableCodec.Items.Clear();
            ListBoxDisableCodec.Items.Clear();

            ListBoxEnableCodec.Items.Add(cCodec.AUDIO_CODEC_NAME_G711U);
            ListBoxEnableCodec.Items.Add(cCodec.AUDIO_CODEC_NAME_G711A);
            ListBoxEnableCodec.Items.Add(cCodec.AUDIO_CODEC_NAME_ILBC);
            ListBoxEnableCodec.Items.Add(cCodec.AUDIO_CODEC_NAME_GSM610);
            ListBoxEnableCodec.Items.Add(cCodec.AUDIO_CODEC_NAME_G729);
        }
                
        //////////////////////////////////////////////////////////////////////////
        //////////////////////////////////////////////////////////////////////////

        private Boolean AddLoginChecks()
        {
            if(ComboLogin.Text.Length == 0)
            {
                MessageBox.Show("Please enter login.", "VaxVoIP", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                return false;
            }
            
            if (!cVaxDefine.IsDigitAll(ComboLogin.Text))
            {
                MessageBox.Show("Login should contain only numerals 0 through 9. For example: 12, 324323, 800122, 2109756 etc.", "VaxVoIP", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                return false;
            }
            
            if(EditPassword.Text.Length == 0)
            {
                MessageBox.Show("Please enter login password.", "VaxVoIP", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                return false;
            }

            if(ComboLogin.Text.Length > 20 || EditPassword.Text.Length > 20)
            {
                MessageBox.Show("Login && password size should not be more than 20.", "VaxVoIP", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                return false;
            }

            if(ComboLogin.Text.Substring(0, 2) == "00")
            {
                MessageBox.Show("Please do not enter 00 as login prefix. 00 prefix is use to prepare dial plan.", "VaxVoIP", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                return false;
            }

            if(ListBoxEnableCodec.Items.Count == 0)
            {
                MessageBox.Show("Please select a codec.", "VaxVoIP", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                return false;
            }

            return true;
        }

        //////////////////////////////////////////////////////////////////////////
        //////////////////////////////////////////////////////////////////////////

        private void BtnDisableCodec_Click(object sender, EventArgs e)
        {
            if (ListBoxEnableCodec.SelectedItems.Count == 0) return;

            ListBoxDisableCodec.Items.Add(ListBoxEnableCodec.SelectedItem.ToString());
            ListBoxEnableCodec.Items.Remove(ListBoxEnableCodec.SelectedItem);
        }

        private void BtnEnableCodec_Click(object sender, EventArgs e)
        {
            if (ListBoxDisableCodec.SelectedItems.Count == 0) return;

            ListBoxEnableCodec.Items.Add(ListBoxDisableCodec.SelectedItem.ToString());
            ListBoxDisableCodec.Items.Remove(ListBoxDisableCodec.SelectedItem);
        }

        //////////////////////////////////////////////////////////////////////////
        //////////////////////////////////////////////////////////////////////////
    }
}