﻿using System;
using System.Collections.Generic;
using System.Collections;
using System.Linq;
using System.Text;

namespace ServerREC
{
    class cVaxUser : cVaxHashTextData
    {
        private static cVaxUserList m_objUserList = null;
        private static int m_nRegisterUserCount = 0;

        private Boolean m_bRegistered = false;
        
        //////////////////////////////////////////////////////////////////////////
        //////////////////////////////////////////////////////////////////////////

        public cVaxUser(String sUserName)
        {
            if (cVaxUser.m_objUserList == null)
                cVaxUser.m_objUserList = new cVaxUserList();
        }

        public static void ResetAll()
        {
            m_nRegisterUserCount = 0;
            cVaxUser.m_objUserList = new cVaxUserList();
        }

        //////////////////////////////////////////////////////////////////////////
        //////////////////////////////////////////////////////////////////////////

        private static cVaxUser GetUser(String sUserName)
        {
            return m_objUserList.GetUser(sUserName);
        }

        //////////////////////////////////////////////////////////////////////////
        //////////////////////////////////////////////////////////////////////////

        public static void OnRegisterUser(String sUserName, String sDomain, String sUserAgentName, String sFromIP, int nFromPort, int nRegId)
        {
            cVaxUser objUser = cVaxUser.GetUser(sUserName);

            if (objUser == null)
            {
                cVaxServerREC.mVaxREC.RejectRegister(nRegId, 404, "Not Found");
                
                String sStatus = sUserName + " " + cVaxDefine.USER_NOT_FOUND;
                cVaxServerREC.mVaxREC.GenStatusLog(sStatus);
                
                return;
            }

            objUser.OnRegister(sUserName, nRegId);
        }

        private void OnRegister(String sUserName, int nRegId)
        {
            cVaxServerREC.mVaxREC.AuthRegister(nRegId);

            String sStatus = sUserName + " " + cVaxDefine.USER_REG_TRYING;
            cVaxServerREC.mVaxREC.GenStatusLog(sStatus);
        }

        //////////////////////////////////////////////////////////////////////////
        //////////////////////////////////////////////////////////////////////////

        public static void OnRegisterUserSuccess(String sUserName, String sFromIP, int nFromPort, int nRegId)
        {
            cVaxUser objUser = cVaxUser.GetUser(sUserName);
            if (objUser == null) return;
            
            objUser.OnRegisterSuccess(sUserName);
        }

        private void OnRegisterSuccess(String sUserName)
        {
            if (m_bRegistered == false)
                m_nRegisterUserCount++;

            m_bRegistered = true;

	        String sStatus = sUserName + " " + cVaxDefine.USER_REG_SUCCESS;
            cVaxServerREC.mVaxREC.GenStatusLog(sStatus);
        }

        //////////////////////////////////////////////////////////////////////////
        //////////////////////////////////////////////////////////////////////////

        public static void OnRegisterUserFailed(String sUserName, String sFromIP, int nFromPort, int nRegId)
        {
            cVaxUser objUser = cVaxUser.GetUser(sUserName);
            if (objUser == null) return;

            objUser.OnRegisterFailed(sUserName);
        }

        private void OnRegisterFailed(String sUserName)
        {
            String sStatus = sUserName + " " + cVaxDefine.USER_REG_FAILED;
            cVaxServerREC.mVaxREC.GenStatusLog(sStatus);
        }

        //////////////////////////////////////////////////////////////////////////
        //////////////////////////////////////////////////////////////////////////

        public static void OnUnRegisterUser(String sUserName)
        {
            cVaxUser objUser = cVaxUser.GetUser(sUserName);
            if (objUser == null) return;

            objUser.OnUnRegister(sUserName);
        }

        private void OnUnRegister(String sUserName)
        {
            if (m_bRegistered == true)
                m_nRegisterUserCount--;
            
            m_bRegistered = false;

            String sStatus = sUserName + " " + cVaxDefine.USER_UNREGISTER;
            cVaxServerREC.mVaxREC.GenStatusLog(sStatus);
        }
                       
        //////////////////////////////////////////////////////////////////////////////
        //////////////////////////////////////////////////////////////////////////////
                     
        public static Boolean AddUser(String sUserName, String sPassword, String sAudioCodecList)
        {
            if (cVaxServerREC.mVaxREC.IsInitialized() == false || (sUserName.Length == 0))
                return false;

            RemoveUser(sUserName);

            if (!cVaxServerREC.mVaxREC.AddUser(sUserName, sPassword, sAudioCodecList))
            {
                return false;
            }

            m_objUserList.AddUser(sUserName);
            cStoreUser.AddUser(sUserName, sPassword, sAudioCodecList);

            return true;
        }

        public static void RemoveUser(String sUserName)
        {
            if (cVaxServerREC.mVaxREC.IsInitialized() == false || (sUserName.Length == 0))
                return;

            cVaxServerREC.mVaxREC.RemoveUser(sUserName);
            m_objUserList.RemoveUser(sUserName);

            cStoreUser.RemoveUser(sUserName);
        }

        public static void LoadUser()
        {
            if (!cVaxServerREC.mVaxREC.IsInitialized())
                return;

            ArrayList objListUsers = new ArrayList();
            cStoreUser.GetUserAll(ref objListUsers);

            if (objListUsers.Count == 0)
                return;

            String sPassword = "", sAudioCodecList = "";
           
            foreach (String sUserName in objListUsers)
            {
                cStoreUser.GetUser(sUserName, ref sPassword, ref sAudioCodecList);
                AddUser(sUserName, sPassword, sAudioCodecList);
            }
        }
                
        public static int GetRegisteredUserCount()
        {
            return m_nRegisterUserCount;
        }
    }
}
