﻿using System;
using System.Collections;
using System.Diagnostics;

namespace ServerREC
{
    public class cVaxServerREC : cVaxServerCOM
    {
        public StatusLogForm m_objStatusLogDlg = null;
        public static cVaxServerREC mVaxREC = null;

        String m_sRecordingFolder;
        
        Boolean m_bInitialized = false;
        cVaxCallSessionList m_objSessionList = null;

        #region Methods

        public cVaxServerREC()
        {
            mVaxREC = this;
            ResetAll();
        }

        //////////////////////////////////////////////////////////////////////////////////
        //////////////////////////////////////////////////////////////////////////////////

        private void ResetAll()
        {
            m_objSessionList = new cVaxCallSessionList();
            cVaxUser.ResetAll();
        }

        //////////////////////////////////////////////////////////////////////////////////
        //////////////////////////////////////////////////////////////////////////////////

        public Boolean StartServer(int nListenPort)
        {
            StopServer();

            base.SetLicenseKey("TRIAL-LICENSE-KEY");

            if (!base.Initialize(""))
                return false;

            if (!base.OpenNetworkUDP("", nListenPort))
            {
                UnInitialize();
                return false;
            }

            if (!base.OpenNetworkTCP("", nListenPort))
            {
                UnInitialize();
                return false;
            }

            if (!base.OpenNetworkTLS("", nListenPort + 1, ""))
            {
                UnInitialize();
                return false;
            }

            // base.AttackDetectScanSIP("demo.vaxvoip.com");
            // base.AttackDetectFloodSIP(1000);
            // base.AttackDetectBruteForceSIP(10, 2);

            base.SetListenPortRangeRTP(40000, 60000);
            base.AudioSessionLost(true, 30); // Enable voice session lost for 30 seconds.
            
            base.AddLine("AnonymousUDP", cVaxServerCOM.VAX_LINE_TYPE_UDP, "", "", "", "", "", "255.255.255.255", -1, "");
            base.AddLine("AnonymousTCP", cVaxServerCOM.VAX_LINE_TYPE_TCP, "", "", "", "", "", "255.255.255.255", -1, "");
            base.AddLine("AnonymousTLS", cVaxServerCOM.VAX_LINE_TYPE_TLS, "", "", "", "", "", "255.255.255.255", -1, "");
                        
            m_bInitialized = true;
                        
            OnInitialize();
            return true;
        }

        public void StopServer()
        {
            m_bInitialized = false;
            
            base.UnInitialize();
            ResetAll();
        }

        //////////////////////////////////////////////////////////////////////////////////
        //////////////////////////////////////////////////////////////////////////////////

        public Boolean IsInitialized()
        {
            return m_bInitialized;
        }

        public Boolean AcceptCallSession(int nSessionId)
        {
            return base.AcceptCallSession(nSessionId, 30);
        }

        //////////////////////////////////////////////////////////////////////////////////
        //////////////////////////////////////////////////////////////////////////////////
        
        public void SetRecordingFolder(String sRecPath)
        {
            m_sRecordingFolder = sRecPath;
        }

        public String GetRecordingFolder()
        {
            return m_sRecordingFolder;
        }

        //////////////////////////////////////////////////////////////////////////////////
        //////////////////////////////////////////////////////////////////////////////////

        public Boolean RecordWaveStartToCallSession(int nSessionId, String sFileName)
        {
	        return base.RecordWaveStartToCallSession(nSessionId, sFileName, cVaxServerCOM.VAX_WAVE_REC_TYPE_STEREO_PCM);
        }

        //////////////////////////////////////////////////////////////////////////////////
        //////////////////////////////////////////////////////////////////////////////////

        #endregion

        //////////////////////////////////////////////////////////////////////////////////
        //////////////////////////////////////////////////////////////////////////////////
        
        #region Events

        //////////////////////////////////////////////////////////////////////////////////
        //////////////////////////////////////////////////////////////////////////////////

        protected override void OnCallSessionCreated(int nSessionId, int nReasonCode)
        {
            m_objSessionList.AddSession(nSessionId);
        }

        protected override void OnCallSessionClosed(int nSessionId, int nReasonCode)
        {
            cVaxCallSession objCallSession = m_objSessionList.GetSession(nSessionId);

            if (objCallSession != null)
                objCallSession.OnCallSessionClosed(nSessionId, nReasonCode);

            m_objSessionList.RemoveSession(nSessionId);
        }

        protected override void OnCallSessionAccessAudioPCM(int nSessionId, int nAccessType, Byte[] aDataPCM, int nSizePCM, int nTypePCM)
        {
            cVaxCallSession objCallSession = m_objSessionList.GetSession(nSessionId);
            if (objCallSession == null) return;

            objCallSession.OnCallSessionAccessAudioPCM(nSessionId, nAccessType, aDataPCM, nSizePCM);
        }

        //////////////////////////////////////////////////////////////////////////////////
        //////////////////////////////////////////////////////////////////////////////////

        protected override void OnIncomingCall(int nSessionId, String sCallerName, String sCallerId, String sCalleeName, String sCalleeId, String sDialNo, int nFromPeerType, String sFromPeerName, String sUserAgentName, String sRecXML, String sFromIP, int nFromPort)
        {
            cVaxCallSession objCallSession = m_objSessionList.GetSession(nSessionId);
            if (objCallSession == null) return;
        
            objCallSession.OnIncomingCall(nSessionId, sCallerName, sCallerId, sCalleeName, sCalleeId, sDialNo, nFromPeerType, sFromPeerName, sUserAgentName, sRecXML, sFromIP, nFromPort);
        }

        protected override void OnCallSessionConnected(int nSessionId)
        {
            cVaxCallSession objCallSession = m_objSessionList.GetSession(nSessionId);
            if (objCallSession == null) return;

            objCallSession.OnCallSessionConnected(nSessionId);
        }

        //////////////////////////////////////////////////////////////////////////////////
        //////////////////////////////////////////////////////////////////////////////////

        protected override void OnLineRegisterTrying(String sLineName)
        {
            cVaxLine.OnLineRegisterTrying(sLineName);
        }

        protected override void OnLineRegisterFailed(String sLineName, int nStatusCode, String sReasonPhrase)
        {
            cVaxLine.OnLineRegisterFailed(sLineName, nStatusCode, sReasonPhrase);
        }

        protected override void OnLineRegisterSuccess(String sLineName)
        {
            cVaxLine.OnLineRegisterSuccess(sLineName);
        }

        //////////////////////////////////////////////////////////////////////////////////
        //////////////////////////////////////////////////////////////////////////////////

        protected override void OnRegisterUser(String sUserName, String sDomain, String sUserAgentName, String sFromIP, int nFromPort, int nRegId)
        {
            cVaxUser.OnRegisterUser(sUserName, sDomain, sUserAgentName, sFromIP, nFromPort, nRegId);
        }

        protected override void OnRegisterUserSuccess(String sUserName, String sFromIP, int nFromPort, int nRegId)
        {
            cVaxUser.OnRegisterUserSuccess(sUserName, sFromIP, nFromPort, nRegId);
        }

        protected override void OnRegisterUserFailed(String sUserName, String sFromIP, int nFromPort, int nRegId)
        {
            cVaxUser.OnRegisterUserFailed(sUserName, sFromIP, nFromPort, nRegId);
        }
                
        protected override void OnUnRegisterUser(String sUserName)
        {    
            cVaxUser.OnUnRegisterUser(sUserName);
        }

        //////////////////////////////////////////////////////////////////////////////////
        //////////////////////////////////////////////////////////////////////////////////
        protected override void OnCallSessionMediaDetectStartedVAD(int nSessionId)
        {
            cVaxCallSession objCallSession = m_objSessionList.GetSession(nSessionId);
            if (objCallSession == null) return;

            objCallSession.OnCallSessionMediaDetectStartedVAD(nSessionId);
        }

        protected override void OnCallSessionMediaDetectCompletedVAD(int nSessionId, Byte[] aDataPCM, int nSizePCM, int nTypePCM)
        {
            cVaxCallSession objCallSession = m_objSessionList.GetSession(nSessionId);
            if (objCallSession == null) return;

            objCallSession.OnCallSessionMediaDetectCompletedVAD(nSessionId, aDataPCM, nSizePCM, nTypePCM);
        }

        protected override void OnCallSessionMediaDetectExpiredVAD(int nSessionId)
        {
            cVaxCallSession objCallSession = m_objSessionList.GetSession(nSessionId);
            if (objCallSession == null) return;

            objCallSession.OnCallSessionMediaDetectExpiredVAD(nSessionId);
        }

        //////////////////////////////////////////////////////////////////////////////////
        //////////////////////////////////////////////////////////////////////////////////

        public void GenStatusLog(String sStatus)
        {
            OnStatusLog(sStatus);
        }

        protected override void OnVaxErrorLog(String sFuncName, int nErrorCode, String sErrorMsg)
        {
            String sMsg = "[Method: " + sFuncName + "] [Error Msg: " + sErrorMsg + "] [Code: " + nErrorCode.ToString() + "]";
            OnStatusLog(sMsg);
        }

        protected override void OnCallSessionErrorLog(int nSessionId, int nErrorCode, String sErrorMsg)
        {
            String sMsg = "[Session: " + nSessionId.ToString() + "] [Error Msg: " + sErrorMsg + "] [Code: " + nErrorCode.ToString() + "]";
            OnStatusLog(sMsg);
        }

        //////////////////////////////////////////////////////////////////////////////////
        //////////////////////////////////////////////////////////////////////////////////
        
        private void OnInitialize()
        {
            if (!cVaxSettingDefault.LoadUserDefault())
            {
                cVaxUser.LoadUser();
            }
            
            if (!cVaxSettingDefault.LoadLineDefault())
            {
                cVaxLine.LoadLine();
            }
        }

        private void OnStatusLog(String sMsg)
        {
            m_objStatusLogDlg.OnStatusLog(sMsg);
        }
                
        #endregion
    }
}
